(*
tools.ml
普通はこの程度は標準だろうという関数
OCaml 4.02
2015.03.13 Takeshi Yoneki
ostra@asahi.email.ne.jp
*)

exception ERROR_BUG

module Unix = struct
  include Unix

  let read_file_all path =
    let sz = (Unix.stat path).Unix.st_size in
    let zmax = 32 * 1024 in
    let ic = open_in_bin path in
    let rec read_block pool rest =
      let bsz = if rest > zmax then zmax else rest in
      let buffer = Bytes.create bsz in
      let rsz = input ic buffer 0 bsz in
      let data =
        if rsz < bsz then
          let result = Bytes.create rsz in
          let _ = String.blit buffer 0 result 0 rsz in
          result
        else
          buffer
      in
      let pool = data :: pool in
      let rest = rest - rsz in
      if rest <= 0 then
        List.rev pool
      else
        read_block pool rest
    in
    let pool = read_block [] sz in
    let _ = close_in ic in
    String.concat "" pool

  let write_file_all path data =
    let oc = open_out path in
    let _ = output oc data 0 (String.length data) in
    close_out oc
end

module Char = struct
  include Char

  let is_alphabet c = 
    (c >= 'a' && 'z' >= c) || (c >= 'A' && 'Z' >= c)
  let is_number c = 
    c >= '0' && '9' >= c
  let is_hex c =
    (c >= 'a' && 'f' >= c) || (c >= 'A' && 'F' >= c) || is_number c
end

module List = struct
  include List

  let take n xs =
    let rec loop n xs ac =
      match xs with
      | [] -> 
        ac
      | (a :: rest) ->
        if n <= 0 then
          ac
        else
          loop (n - 1) rest (a :: ac)
    in
    List.rev (loop n xs [])

  let drop n xs =
    let rec loop n xs =
      match xs with
      | [] -> 
        []
      | (a :: rest) ->
        if n <= 0 then
          xs
        else
          loop (n - 1) rest
    in
    loop n xs

  let divide n xs =
    let rec loop n xs ac = 
      match xs with
      |  [] -> 
        ac
      | _ ->
        let ls = take n xs in
        let rest = drop n xs in
        loop n rest (ls :: ac)
    in
    List.rev (loop n xs [])

  let is_empty xs =
    xs = []
    
  let filter_map fx xs =
    let r =
      List.fold_left
        (fun ac x ->
          match fx x with
          | Some a -> a :: ac
          | _ -> ac
        )
        []
        xs
    in
    List.rev r

  let range n =
    let rec loop ac a =
      let ac = a :: ac in
      if a <= 0 then
        ac
      else
        loop ac (a - 1)
    in
    loop [] (n - 1)

  let last xs =
    let rec loop xs =
      if List.length xs = 1 then
        List.hd xs
      else
        loop (List.tl xs)
    in
    loop xs

  let maximum xs =
    let rec loop ac xs =
      if xs = [] then
        ac
      else
        let x = List.hd xs in
        loop (if x > ac then x else ac) (List.tl xs)
    in
    loop (List.hd xs) (List.tl xs)
end

module CharList = struct
  let of_string_rev s =
    let cl = ref [] in
    let _ = String.iter (fun c -> cl := c :: !cl) s in
    !cl

  let of_string s =
    List.rev (of_string_rev s)

  let to_string_rev cl =
    let sz = List.length cl in
    let bf = Bytes.create sz in
    let _ = List.iteri (fun i ch -> Bytes.set bf (sz - i - 1) ch) cl in
    bf

  let to_string cl =
    let bf = Bytes.create (List.length cl) in
    let _ = List.iteri (fun i ch -> Bytes.set bf i ch) cl in
    bf

  let starts_with s pfx = 
    if s = [] || pfx = [] then
      false
    else
      let rec loop s pfx =
        if pfx = [] then
          true
        else if s = [] then
          false
        else if List.hd s <> List.hd pfx then
          false
        else
          loop (List.tl s) (List.tl pfx)
      in
      loop s pfx

  let split_n dlmt n s =
    let rec loop respSize resp cur rest =
      if rest = [] then
        if cur <> [] then
          cur :: resp
        else
          resp
      else if n > 0 && respSize = (n - 1) then
        (List.rev rest) :: resp
      else
        let hit = 
          List.fold_left
            (fun ac d ->
              if ac <> [] then
                ac
              else if starts_with rest d then 
                d 
              else 
                ac
            )
            []
            dlmt
        in
        let rpsz = ref respSize in
        let rp = ref resp in
        let cr = ref cur in
        let rt = ref rest in
        let _ =
          if hit <> [] then
            let _ = cr := [] in
            let _= rt := List.fold_left (fun ac _ -> List.tl ac) rest hit in
            if cur <> [] then
              let _ = rp := cur :: resp in
              rpsz := respSize + 1
            else
              ()
          else
            let _ = cr := (List.hd rest) :: cur in
            rt := List.tl rest
        in
        loop !rpsz !rp !cr !rt
    in
    let resp = loop 0 [] [] s in
    List.rev (List.map (fun r -> List.rev r) resp)

  let split dlmt s =
    split_n dlmt (-1) s
end

module String = struct
  include String

  let ends_with s sfx = 
    CharList.starts_with (CharList.of_string_rev s) (CharList.of_string_rev sfx)
  
  let starts_with s pfx = 
    CharList.starts_with (CharList.of_string s) (CharList.of_string pfx)

  let is_empty s =
    s = ""

  (* quick search (Sunday法) *)
  let mk_qs_table p =
    let pl = String.length p in
    let tbl = Array.make 256 (1 + pl) in
    let _ = 
      String.iteri 
        (fun i ch -> 
          tbl.(Char.code ch) <- pl - i
        ) 
        p 
    in
    (p, tbl)

  let qs_find t (p, tbl) =
    let tl = String.length t in
    let pl = String.length p in
    let imax = tl - pl in
    if imax < 0 || pl = 0 then
      -1
    else
      let rec loop_i i =
        let rec loop_j j =
          if j = pl || t.[i + j] <> p.[j] then 
            j 
          else 
            loop_j (j + 1)
        in
        if i > imax then
          -1
        else if loop_j 0 = pl then 
          i
        else
          let n = i + pl in
          if n >= tl then
            -1
          else 
            loop_i (i + tbl.(Char.code t.[n]))
      in
      loop_i 0

  let find t p =
    qs_find t (mk_qs_table p)

  let qs_replace t (p, tbl) q =
    let pl = String.length p in
    let rec loop t ac =
      let n = qs_find t (p, tbl) in
      if n = -1 then
        t :: ac
      else
        let tl = String.length t in
        let th = String.sub t 0 n in
        let tr = String.sub t (n + pl) (tl - (n + pl)) in
        loop tr (q :: th :: ac)
    in
    let r = loop t [] in
    String.concat "" (List.rev r)
 
  let replace t p q =
    qs_replace t (mk_qs_table p) q

  let fold_left fn ac s =
    let cl = CharList.of_string s in
    List.fold_left fn ac cl

  let split_n dlmt n s =
    let cdlmt = List.map (fun d -> CharList.of_string d) dlmt in
    let cs = CharList.of_string s in
    let cr = CharList.split_n cdlmt n cs in
    List.map (fun r -> CharList.to_string r) cr

  let split dlmt s =
    split_n dlmt (-1) s

end

module Unicode = struct
  let ucs4_to_utf8 u =
    let cl = 
      if u < 0x80 then
        [u]
      else if u < 0x800 then
        [ ((u lsr 6) land 0x1F) lor 0xC0; 
          (u land 0x3F) lor 0x80]
      else if u < 0x10000 then
        [ ((u lsr 12) land 0xF) lor 0xE0;
          ((u lsr 6) land 0x3F) lor 0x80;
          (u land 0x3F) lor 0x80]
      else if u < 0x200000 then
        [ ((u lsr 18) land 0x7) lor 0xF0;
          ((u lsr 12) land 0x3F) lor 0x80;
          ((u lsr 6) land 0x3F) lor 0x80;
          (u land 0x3F) lor 0x80]
      else
        [int_of_char '_']
    in
    let bf = Bytes.create (List.length cl) in
    let _ = List.iteri (fun i ch -> Bytes.set bf i (Char.chr ch)) cl in
    bf
end

module Para = struct
  exception ERROR_PARA

  let map fn ls =
    let results = Array.make (List.length ls) None in
    (* go *)
    let thrs =
      List.mapi
        (fun i s ->
          Thread.create
            (fun s ->
              results.(i) <- (Some (fn s))
            )
            s
        )
        ls
    in
    (* wait *)
    List.mapi
      (fun i thr -> 
        let _ = Thread.join thr in
        match results.(i) with
        | Some r -> r
        | _ -> raise ERROR_PARA
      ) 
      thrs 
end
