(*
http://tv.so-net.ne.jp/chart/
テレビ王国のデータをフィルタ出力
2009.09.29 Takeshi Yoneki
XML化してから解析
2011.02.10 Takeshi Yoneki
OCamlに移植
2015.04.02 Takeshi Yoneki
*)

open Tools

let genres = 
  ["cell-genre-107000"; "cell-genre-106000"]

let urlBase = "http://tv.so-net.ne.jp"
let tops = [
  ([("Cookie", "gtv.stationAreaId=24")], urlBase ^ "/chart/", "ｔｖｋ", 'C');
  ([("Cookie", "gtv.stationAreaId=29")], urlBase ^ "/chart/", "テレ玉", 'C');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/", "", 'C');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/bs1.action", "", 'C');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/bs2.action", "", 'C');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/bs3.action", "", 'C');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/bs4.action", "", 'C');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/cs/400624.action", "", 'D');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/cs/400631.action", "", 'D');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/cs/400632.action", "", 'D');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/cs/400633.action", "", 'D');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/cs/400660.action", "", 'D');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/cs/400669.action", "", 'D');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/cs/400611.action", "", 'D');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/cs/400614.action", "", 'D');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/cs/400615.action", "", 'D');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/cs/400616.action", "", 'D');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/cs/400617.action", "", 'D');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/cs/400619.action", "", 'D');
  ([("Cookie", "gtv.stationAreaId=23")], urlBase ^ "/chart/cs/400664.action", "", 'D')
]

let rec scanChart rE reqStation listMode recordSet =
  let st = ref "" in
  let _ = 
    if listMode = 'D' then
      let tts = HtmlParser.get_elements_by_tag_name rE "title" in
      let ts = 
        if not (List.is_empty tts) then
          let tx = HtmlParser.get_element_text (List.hd tts) in
          if not (String.is_empty tx) then
            List.hd (String.split [" "; "("; "　"] tx)
          else
            ""
        else
          ""
      in      
      st := ts
    else
      ()
  in
  let _ =
    let t = 
      List.fold_left
        (fun ac h1E ->
          if String.is_empty ac && HtmlParser.get_attribute h1E "class" = "chartTitle" then
            String.trim (String.concat "" (String.split ["\r"; "\n"] (HtmlParser.get_element_text h1E)))
          else
            ac
        )
        ""
        (HtmlParser.get_elements_by_tag_name rE "h1")
    in
    prerr_endline t
  in
  let _ =
    List.iter
      (fun divE ->
        let classA = HtmlParser.get_attribute divE "class" in
        let _ = 
          if listMode = 'C' && (String.find classA "cell-top") <> -1 && (String.find classA "cell-station") <> -1 then
            let tx = HtmlParser.get_attribute divE "title" in
            st := tx
          else
            ()
        in
        let gs = List.filter (fun g -> String.find classA g <> -1) genres in
        let gen = if List.length gs > 0 then List.hd gs else "" in
        let nTarget = List.maximum (List.mapi (fun i g -> if g = gen then i else -1) genres) in
        if nTarget >= 0 then
          let ts = List.filter (fun t -> String.find t "system-cell-schedule-head" <> -1) (String.split [" "] classA) in
          let tx = if not (List.is_empty ts) then List.hd ts else "" in
          let dt = if not (String.is_empty tx) then List.last (String.split ["-"] tx) else "" in
          if String.is_empty reqStation || reqStation = !st then
            let _ =
              List.iter
                (fun spanE ->
                  let spanClassA = HtmlParser.get_attribute spanE "class" in
                  if String.find spanClassA "schedule-title" <> -1 then
                    let tt = HtmlParser.get_element_text spanE in
                    let ls = recordSet.(nTarget) in
                    let ls = (dt, tt, !st) :: ls in
                    recordSet.(nTarget) <- ls
                  else
                    ()
                )
                (HtmlParser.get_elements_by_tag_name divE "span")
            in
            ()
          else
            ()
        else
          ()
      )
      (HtmlParser.get_elements_by_tag_name rE "div")
  in
  recordSet

let getNextUrl url elm listMode =
  let bNext = 
    not (List.is_empty
      (List.filter
        (fun liE ->
          let classA = HtmlParser.get_attribute liE "class" in
          String.find classA "arrow bdAj" <> -1
        ) 
        (HtmlParser.get_elements_by_tag_name elm "li")))
  in
  if listMode = 'D' || bNext then
    let ax =
        (List.filter
          (fun aE ->
            not (List.is_empty
              (List.filter
                (fun imgE ->
                  let src = HtmlParser.get_attribute imgE "src" in
                  String.find src "btn_arrow_j" <> -1
                )
                (HtmlParser.get_elements_by_tag_name aE "img")))
          )
          (HtmlParser.get_elements_by_tag_name elm "a"))
    in
    let next =
      if not (List.is_empty ax) then
        (List.hd (String.split ["?"] url)) ^ (HtmlParser.get_attribute (List.hd ax) "href")
      else
        ""
    in
    if not (String.is_empty next) && listMode = 'D' then
      if next < url then
        ""
      else
        next
    else
      next
  else
    ""

let mkLogPath reqHead url =
  let n, v = List.hd reqHead in
  let t = String.replace (v ^ url) urlBase "" in
  let t = String.replace t "/" "_" in
  let t = String.replace t "?" "_" in
  let t = String.replace t "=" "_" in
  let t = String.replace t "." "_" in
  "log/" ^ t ^ ".txt"

let onProc (reqHead, url, reqStation, listMode) recordSet =
  let prevUrls = ref [] in
  let rec loop recordSet url =
    if List.mem url !prevUrls then
      recordSet
    else
      let _ = prevUrls := url :: !prevUrls in
      let rE =
        List.fold_left
          (fun e i ->
            if e = HtmlParser.Empty then
              let _ = 
                if i > 0 then
                  let _ = Unix.sleep i in
                  prerr_endline (url ^ "*")
                else
                  ()
              in
              let t = Http.read_all reqHead url in
              let e = HtmlParser.parse t in
              (*let logPath = mkLogPath reqHead url in
              let _ = Unix.write_file_all logPath t in
              let _ = Unix.write_file_all (logPath ^ ".xml") (HtmlParser.to_xml_string e) in*)
              let test e = 
                List.fold_left
                  (fun ac h1E ->
                    if not ac && HtmlParser.get_attribute h1E "class" = "chartTitle" then
                      not (String.is_empty (HtmlParser.get_element_text h1E))
                    else
                      ac
                  )
                  false
                  (HtmlParser.get_elements_by_tag_name e "h1")
              in
              if test e then e else HtmlParser.Empty
            else
              e
          )
          HtmlParser.Empty
          (List.range 10)
      in
      if rE = HtmlParser.Empty then
        let _ = prerr_endline "NG" in
        let (n, v) = List.hd reqHead in
        let _ = recordSet.(0) <- ("000000000000", url, v) :: recordSet.(0) in
        recordSet
      else
        let recordSet = scanChart rE reqStation listMode recordSet in
        let next = getNextUrl url rE listMode in
        if String.is_empty next then
          recordSet
        else
          loop recordSet next
  in
  let _ = prerr_endline url in
  loop recordSet url

let outputRecord (dt, tt, st) =
  let y = String.sub dt 0 4 in
  let m = String.sub dt 4 2 in
  let d = String.sub dt 6 2 in
  let h = String.sub dt 8 2 in
  let n = String.sub dt 10 2 in
  let ss = y ^ "." ^ m ^ "." ^ d ^ " " ^ h ^ ":" ^ n ^ "\t" ^ tt ^ "(" ^ st ^ ")" in
  print_endline ss

let compareRecord a b =
  let (dt_a, tt_a, st_a) = a in
  let (dt_b, tt_b, st_b) = b in
  if dt_a = dt_b then
    if st_a = st_b then
      String.compare tt_a tt_b
    else
      String.compare st_a st_b
  else
    String.compare dt_a dt_b

let main () =
  let rx =
    List.map
      (fun ts ->
        List.map
          (fun t ->
            let rs = Array.of_list (List.map (fun g -> []) genres) in
            onProc t rs
          )
          ts
      )
      (List.divide 4 tops)
  in
  let rs = Array.of_list (List.map (fun g -> []) genres) in
  let _ = 
    List.iter
      (fun a -> Array.iteri (fun i r -> rs.(i) <- r @ rs.(i)) a) 
      (List.concat rx)
  in
  let nonTargets = ["ＷＯＷＯＷ"; "J:COMチャンネル"; "AT-X"; "スター・チャンネル"] in
  let isTarget st =
    List.is_empty (List.filter (fun t -> String.find st t <> -1) nonTargets)
  in
  let _ =
    Array.iter
      (fun gm ->
        List.iter
          (fun (dt, tt, st) ->
            if isTarget st then
              outputRecord (dt, tt, st)
            else
              ()
          )
          (List.sort_uniq compareRecord gm)
      )
      rs
  in
  ()

(*
let main1 () =
  let t = Http.read_all [] "http://tv.so-net.ne.jp/chart/cs/400619.action" in
  let e = HtmlParser.parse t in
  let x = HtmlParser.to_xml_string e in
  let _ = Unix.write_file_all "t.xml" x in
  ()

let main2 () =
  let t = Unix.read_file_all "sample11.html" in
  let e = HtmlParser.parse t in
  let x = HtmlParser.to_xml_string e in
  let _ = Unix.write_file_all "sample11.xml" x in
  ()

let main () =
  let d = Unix.read_file_all "20150330a.txt" in
  let a = String.find d "ＷＯＷＯＷ" in
  let _ = print_endline (string_of_int a) in
  ()
*)

let _ = main ()
